# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////
from typing import Dict, List

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import (
    Qt,
    Signal,
    QPropertyAnimation,
    QEasingCurve,
)
from PySide6.QtWidgets import (
    QFrame,
    QSizePolicy,
    QGridLayout,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////
from .framed_label import FramedLabel
from ...kernel.app_settings import Settings

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> FUNCTIONS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////

# CLASS
# ///////////////////////////////////////////////////////////////


class ToggleRadio(QFrame):
    clicked = Signal()

    # ///////////////////////////////////////////////////////////////
    def __init__(
        self, items: List[str], default: str = "", parent=None, *args, **kwargs
    ) -> None:
        super().__init__(parent, *args, **kwargs)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        # //////
        self._value = ""
        self._options_list = items
        self._default = default
        self._options: Dict[str, FramedLabel] = {}
        # //////
        self.initUI()

    # ///////////////////////////////////////////////////////////////

    def initUI(self) -> None:
        self.grid = QGridLayout(self)
        self.grid.setObjectName("grid")
        self.grid.setSpacing(0)
        self.grid.setContentsMargins(4, 4, 4, 4)
        self.grid.setAlignment(Qt.AlignmentFlag.AlignCenter)

        # //////
        self.selector = QFrame(self)
        self.selector.setObjectName("selector")

        # //////
        for option in self._options_list:  # Exemple avec trois labels
            self.add_option(option_text=option)

    # ///////////////////////////////////////////////////////////////

    def initialize_selector(self, default: str = "") -> None:
        self._default = default
        # //////
        if self._options:
            # //////
            first_option_key = next(iter(self._options_list), None)
            selected_option = self._options.get(
                self._default.capitalize(), self._options.get(first_option_key)
            )
            default_pos = self.grid.indexOf(selected_option)

            # //////
            self.grid.addWidget(self.selector, 0, default_pos)
            self.selector.lower()  # S'assure que le selector reste en dessous
            self.selector.update()  # Force le rafraîchissement si nécessaire

    # ///////////////////////////////////////////////////////////////

    def add_option(self, option_text: str) -> None:
        option = FramedLabel(option_text, self)
        option.setObjectName(f"opt_{option_text}")
        option.setFrameShape(QFrame.NoFrame)
        option.setFrameShadow(QFrame.Raised)
        option.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        # //////
        self.grid.addWidget(option, 0, len(self._options.items()))

        # //////
        option.mousePressEvent = lambda event, option=option: self.toggle_selection(
            option
        )
        self._options[option_text] = option

    # ///////////////////////////////////////////////////////////////

    def get_value_option(self) -> FramedLabel | None:
        return self._options.get(self._value)

    # ///////////////////////////////////////////////////////////////

    def toggle_selection(self, option: FramedLabel) -> None:
        self._value = option.text()
        # //////
        self.clicked.emit()
        self.move_selector(option)

    # ///////////////////////////////////////////////////////////////

    def move_selector(self, option: FramedLabel) -> None:

        # Variables d'animation
        start_pos = self.selector.pos()
        end_pos = option.pos()
        # //////
        self.selector.setGeometry(option.geometry())
        self.selector.lower()  # S'assure que le selector reste en dessous
        self.selector.update()  # Force le rafraîchissement si nécessaire

        # ANIMATION
        self.selector_animation = QPropertyAnimation(self.selector, b"pos")
        self.selector_animation.setDuration(Settings.Gui.TIME_ANIMATION)
        self.selector_animation.setStartValue(start_pos)
        self.selector_animation.setEndValue(end_pos)
        self.selector_animation.setEasingCurve(QEasingCurve.InOutQuart)
        self.selector_animation.start()



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import (
    Qt,
    QSize,
)
from PySide6.QtGui import (
    QIcon,
)
from PySide6.QtWidgets import (
    QHBoxLayout,
    QLabel,
    QToolButton,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////
from ...kernel.app_components import SizePolicy

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> FUNCTIONS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////

# CLASS
# ///////////////////////////////////////////////////////////////


class IconButton(QToolButton):
    def __init__(self, parent=None, *args, **kwargs) -> None:
        super().__init__(parent, *args, **kwargs)
        self.setProperty("type", "IconButton")
        self.setContentsMargins(10, 0, 0, 0)

        self.icon_label = QLabel()
        self.text_label = QLabel()

        # Appliquer des styles au QLabel pour gérer l'alignement du texte
        self.text_label.setAlignment(
            Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter
        )
        self.text_label.setWordWrap(True)
        self.text_label.setStyleSheet("background-color: rgba(0, 0, 0, 0);")

        # Configuration de la disposition
        layout = QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(10)  # Espace entre l'icône et le texte
        layout.setAlignment(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
        layout.addWidget(self.icon_label)
        layout.addWidget(self.text_label)

        # Configurer le bouton pour s'étendre
        self.setSizePolicy(SizePolicy.H_EXPANDING_V_EXPANDING)

        # //////
        self.text_label.hide()

    # ///////////////////////////////////////////////////////////////

    def setIcon(self, icon_path) -> None:
        icon = icon_path if isinstance(icon_path, QIcon) else QIcon(icon_path)
        self.icon_label.setPixmap(
            icon.pixmap(QSize(24, 24))
        )  # Ajuster la taille de l'icône
        self.icon_label.setFixedSize(24, 24)  # Fixer la taille du QLabel de l'icône
        self.icon_label.setStyleSheet("background-color: rgba(0, 0, 0, 0);")

    # ///////////////////////////////////////////////////////////////

    def setText(self, text) -> None:
        self.text_label.setText(text)

    # ///////////////////////////////////////////////////////////////

    def setTextAlignment(self, alignment) -> None:
        self.text_label.setAlignment(alignment)

    # ///////////////////////////////////////////////////////////////

    def setSpacing(self, spacing) -> None:
        layout = self.layout()
        layout.setSpacing(spacing)

    # ///////////////////////////////////////////////////////////////

    def sizeHint(self) -> QSize:
        return QSize(100, 40)  # Taille suggérée du bouton

    # ///////////////////////////////////////////////////////////////

    def toggleTextVisibility(self) -> None:
        if self.text_label.isHidden():
            self.text_label.show()
        # //////
        else:
            self.text_label.hide()



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////
import sys
import platform
from pathlib import Path

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////
from .kernel import *
from .widgets import *

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////
os_name = platform.system()
widgets = None

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////
APP_PATH = Path(getattr(sys, "_MEIPASS", Path(sys.argv[0]).resolve().parent))
# //////
_dev = True if not hasattr(sys, "frozen") else False

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class EzQt_App(QMainWindow):
    def __init__(
        self,
        themeFileName: str = None,
    ) -> None:
        QMainWindow.__init__(self)

        # ==> KERNEL LOADER
        # ///////////////////////////////////////////////////////////////.
        Kernel.loadFontsResources()
        Kernel.loadAppSettings()

        # ==> INITIALIZE COMPONENTS
        # ///////////////////////////////////////////////////////////////.
        Fonts.initFonts()
        SizePolicy.initSizePolicy()

        # ==> SET AS GLOBAL WIDGETS
        # ///////////////////////////////////////////////////////////////
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        global widgets
        widgets = self.ui

        # ==> USE CUSTOM TITLE BAR | "True" for Windows
        # ///////////////////////////////////////////////////////////////
        Settings.App.ENABLE_CUSTOM_TITLE_BAR = True if os_name == "Windows" else False

        # ==> APP DATA
        # ///////////////////////////////////////////////////////////////
        self.setWindowTitle(Settings.App.NAME)
        (
            self.setAppIcon(Images.logo_placeholder, yShrink=0)
            if Settings.Gui.THEME == "dark"
            else self.setAppIcon(Images.logo_placeholder, yShrink=0)
        )

        # ==> TOGGLE MENU
        # ///////////////////////////////////////////////////////////////
        widgets.menuContainer.toggleButton.clicked.connect(
            lambda: UIFunctions.toggleMenuPanel(self, True)
        )

        # ==> TOGGLE SETTINGS
        # ///////////////////////////////////////////////////////////////
        widgets.headerContainer.settingsTopBtn.clicked.connect(
            lambda: UIFunctions.toggleSettingsPanel(self, True)
        )

        # SET UI DEFINITIONS
        # ///////////////////////////////////////////////////////////////
        UIFunctions.uiDefinitions(self)

        # SET THEME
        # ///////////////////////////////////////////////////////////////
        self._themeFileName = themeFileName
        UIFunctions.theme(self, self._themeFileName)
        # //////
        _theme = Kernel.loadKernelConfig("app")["theme"]
        self.ui.themeToggleButton.initialize_selector(_theme)
        self.ui.headerContainer.update_all_theme_icons()
        self.ui.menuContainer.update_all_theme_icons()
        # //////
        self.ui.themeToggleButton.clicked.connect(self.setAppTheme)

    # SET APP THEME
    # ///////////////////////////////////////////////////////////////
    def setAppTheme(self) -> None:
        theme = self.ui.themeToggleButton._value
        Settings.Gui.THEME = theme.lower()
        Kernel.writeYamlConfig(keys=["app", "theme"], val=theme.lower())
        # //////
        QTimer.singleShot(100, self.updateUI)

    # UPDATE UI
    # ///////////////////////////////////////////////////////////////
    def updateUI(self) -> None:
        new_pos = self.ui.themeToggleButton.get_value_option()
        self.ui.themeToggleButton.move_selector(new_pos)

        # //////
        UIFunctions.theme(self, self._themeFileName)
        # //////
        EzApplication.instance().themeChanged.emit()
        self.ui.headerContainer.update_all_theme_icons()
        self.ui.menuContainer.update_all_theme_icons()

        # //////
        QApplication.processEvents()

    # SET APP ICON
    # ///////////////////////////////////////////////////////////////
    def setAppIcon(
        self, icon: str | QPixmap, yShrink: int = 0, yOffset: int = 0
    ) -> None:
        return self.ui.headerContainer.set_app_logo(
            logo=icon, y_shrink=yShrink, y_offset=yOffset
        )

    # ADD MENU & PAGE
    # ///////////////////////////////////////////////////////////////
    def addMenu(self, name: str, icon: str) -> QWidget:
        page = self.ui.pagesContainer.add_page(name)
        # //////
        menu = self.ui.menuContainer.add_menu(name, icon)
        menu.setProperty("page", page)
        if len(self.ui.menuContainer.menus) == 1:
            menu.setProperty("class", "active")
        # //////
        menu.clicked.connect(
            lambda: widgets.pagesContainer.stackedWidget.setCurrentWidget(page)
        )
        menu.clicked.connect(self.switchMenu)

        # //////
        return page

    # MENU SWITCH
    # ///////////////////////////////////////////////////////////////
    def switchMenu(self) -> None:
        # GET BUTTON CLICKED
        sender = self.sender()
        senderName = sender.objectName()

        # SHOW HOME PAGE
        for btnName, btnWidget in self.ui.menuContainer.menus.items():
            if senderName == f"menu_{btnName}":
                UIFunctions.deselectMenu(self, senderName)
                UIFunctions.selectMenu(self, senderName)

    # RESIZE EVENTS
    # ///////////////////////////////////////////////////////////////
    def resizeEvent(self, event) -> None:
        # Update Size Grips
        UIFunctions.resize_grips(self)

    # MOUSE CLICK EVENTS
    # ///////////////////////////////////////////////////////////////
    def mousePressEvent(self, event) -> None:
        # SET DRAG POS WINDOW
        self.dragPos = event.globalPosition().toPoint()

        # //////
        if _dev:
            # PRINT OBJECT NAME
            # //////
            child_widget = self.childAt(event.position().toPoint())
            if child_widget:
                child_name = child_widget.objectName()
                print(child_name)

            # PRINT MOUSE EVENTS
            # //////
            elif event.buttons() == Qt.LeftButton:
                print(f"Mouse click: LEFT CLICK")
            elif event.buttons() == Qt.RightButton:
                print("Mouse click: RIGHT CLICK")



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# ICON BUTTON
from .icon_button import IconButton

# FRAMED LABEL
from .framed_label import FramedLabel

# TOGGLE RADIO
from .toggle_radio import ToggleRadio
